import { EventPost, OnlineStatus } from "../../protocol/payload/events"
import { FuncCall, FuncCallRes } from "../../protocol/payload/functions"
import { PropGet, PropGetRes, PropPost, PropSet, PropSetRes } from "../../protocol/payload/properties"
import { ThingModelPost } from "../../protocol/payload/thingmodel"
import { logger } from "../../util/log"
import { deviceEntity } from "./device"


// 实体模型 上下线状态上报
function entityOnlineStatusPost(isOnline: boolean): Error | void {
	logger.info("Enter Device entityOnlineStatusPost")
	let post: OnlineStatus = {
		deviceId: deviceEntity.deviceId,
		events: isOnline ? "connected" : "disconnected",
		model: deviceEntity.thingModel
	}

	// 上报
	deviceEntity.sdk.OnlineStatusPost("device", "", post)
	logger.info("Leave Device entityOnlineStatusPost")
	return
}

// 实体模型 物模型上报
function entityThingModelPost(): Error | void {
	logger.info("Enter Device entityThingModelPost")
	let post: ThingModelPost = {
		deviceId: deviceEntity.deviceId,
		"thing-model": deviceEntity.thingModel
	}

	// 上报
	deviceEntity.sdk.ThingModelPost("device", "", post)
	logger.info("Leave Device entityThingModelPost")
	return
}

// 通用属性 上报
function entityPropertiesPost(): Error | void {
	logger.info("Enter Device entityPropertiesPost")
	let post: PropPost = {
		deviceId: deviceEntity.deviceId,
		version: deviceEntity.version,
		properties: { ...(deviceEntity.prop || {}) }
	}

	// 上报
	deviceEntity.sdk.PropertiesPost("device", "", post)
	logger.info("Leave Device entityPropertiesPost")
	return
}

// 通用属性设置
function entityPropertiesSet(inSet: PropSet): Error | void {
	logger.info("Enter Device entityPropertiesSet")
	// 设置
	deviceEntity.prop = { ...inSet.properties }
	// response
	let outRes: PropSetRes = {
		deviceId: inSet.deviceId,
		messageId: inSet.messageId,
		version: inSet.version == 65535 ? 0 : (inSet.version + 1),
		code: 200,
		properties: { ...deviceEntity.prop }
	}

	deviceEntity.sdk.PropertiesSetResponse("device", "", outRes)
	logger.info("Leave Device entityPropertiesSet")
	return
}

// 通用属性获取
function entityPropertiesGet(inGet: PropGet): Error | void {
	logger.info("Enter Device entityPropertiesGet")

	// response
	let outRes: PropGetRes = {
		deviceId: inGet.deviceId,
		messageId: inGet.messageId,
		version: deviceEntity.version,
		code: 200,
		properties: {}
	}

	let lenProp = inGet.properties.length
	if (lenProp > 0 && deviceEntity.prop) {
		for (let ii = 0; ii < lenProp; ii++) {
			outRes.properties[inGet.properties[ii]] = deviceEntity.prop[inGet.properties[ii]]
		}
	} else {
		// 获取所有
		outRes.properties = { ...(deviceEntity.prop || {}) }
	}

	deviceEntity.sdk.PropertiesGetResponse("device", "", outRes)
	logger.info("Leave Device entityPropertiesGet")
	return
}

// 通用属性获取
function entityFunctionsCall(inCall: FuncCall): Error | void {
	logger.info("Enter Device entityFunctionsCall")

	// response
	let outRes: FuncCallRes = {
		deviceId: inCall.deviceId,
		messageId: inCall.messageId,
		code: 200,
		functions: []
	}
	for (let i = 0, j = inCall.functions.length; i < j; i++) {
		outRes.functions.push({
			key: inCall.functions[i].key,
			out: inCall.functions[i].in
		})
	}

	deviceEntity.sdk.FunctionsCallResponse("device", "", outRes)

	logger.info("Leave Device entityFunctionsCall")
	return
}

// 通用事件 上报
function entityEventPost(): Error | void {
	logger.info("Enter Device entityEventPost")
	let post: EventPost = {
		deviceId: deviceEntity.deviceId,
		events: [
			{
				key: "event_test",
				parameters: {
					"param": "this is a test event"
				}
			}
		]
	}

	// 上报
	deviceEntity.sdk.EventsPost("device", "", post)
	logger.info("Leave Device entityEventPost")
	return
}


export default {
	entityOnlineStatusPost,
	entityThingModelPost,
	entityPropertiesPost,
	entityPropertiesSet,
	entityPropertiesGet,
	entityFunctionsCall,
	entityEventPost
}

