import joylink, { JoylinkConfig, JoylinkSDK, JoylinkSDKEntity } from "../core/joylink"
import path from "path";
import device from "./device/device"
var fs = require('fs');

// 加载配置文件
function LoadConfigFromFile(fpath: string): JoylinkConfig {
	fpath = path.join(__dirname, fpath)
	let configStr = fs.readFileSync(fpath, 'utf8')
	let cfg: JoylinkConfig = JSON.parse(configStr)
	return cfg;
}

// 主函数
export function main() {
	// 加载joylink配置文件
	let joyConfig: JoylinkConfig = LoadConfigFromFile("res/joylink_config.json")

	// 创建 SDK
	let joySDK: JoylinkSDK = joylink.NewSDK(joyConfig)

	joySDK.Welcome()
	// 初始化 SDK
	joySDK.Initialise()

	// 设置设备回调函数
	joySDK.SetDevCallBackConnect(device.CallBackOnConnect)
	joySDK.SetDevCallBackThMdPostRes(device.CallBackOnThingModelPostRes)
	joySDK.SetDevCallBackPropSet(device.CallBackOnPropertiesSet)
	joySDK.SetDevCallBackPropGet(device.CallBackOnPropertiesGet)
	joySDK.SetDevCallBackFuncCall(device.CallBackOnFunctionsCall)

	// 初始化设备
	device.Initialise(joyConfig.deviceId, joySDK)

	// SDK 连接
	joySDK.Connect()

}
