// 消息主题前缀与版本定义
var prefix: string = "$iot/v1/"
var maxParts: number = 8
var minParts: number = 5

// 消息类型
const TYPE_NAME_PROP: string = "properties";
const TYPE_NAME_EVT: string = "events";
const TYPE_NAME_FUNC: string = "functions";
const TYPE_NAME_REG: string = "register";
const TYPE_NAME_AUTH: string = "auth";
const TYPE_NAME_HB: string = "heartbeat";
const TYPE_NAME_THMD: string = "thing-model";

// 消息主题列表枚举
const AUTH_POST: number = 0;
const AUTH_POST_RES: number = 1;
const THINGMODEL_POST: number = 2;
const THINGMODEL_POST_RES: number = 3;
const EVT_ONLINE: number = 4;
const EVT_OFFLINE: number = 5;
const HB_POST: number = 6;
const HB_POST_RES: number = 7;
const HB_SET: number = 8;
const EVT_POST: number = 9;
const PROP_SET: number = 10;
const PROP_SET_RES: number = 11;
const PROP_GET: number = 12;
const PROP_GET_RES: number = 13;
const PROP_POST: number = 14;
const FUNC_CALL: number = 15;
const FUNC_CALL_RES: number = 16;
const REG: number = 17;
const REG_RES: number = 18;

// 消息主题结构体定义
export type Topic = {
    retain: string;
    version: string;
    objName: string;
    deviceId: string;
    serviceKey: string;
    typeName: string;
    command: string;
    response: string;
}

// 获取消息主题字符串
function Get(tType: number, objName: string, deviceId: string | undefined, servicKey: string): string {
    var retTopic: string
    var tailStr: string
    switch (tType) {
        case AUTH_POST:
            tailStr = "/auth/post"; break;
        case AUTH_POST_RES:
            tailStr = "/auth/post/response"; break;
        case EVT_ONLINE:
            tailStr = "/events/online"; break;
        case EVT_OFFLINE:
            tailStr = "/events/offline"; break;
        case HB_POST:
            tailStr = "/hearbeat/post"; break;
        case HB_POST_RES:
            tailStr = "/hearbeat/post/response"; break;
        case HB_SET:
            tailStr = "/hearbeat/set"; break;
        case EVT_POST:
            tailStr = "/events/post"; break;
        case PROP_SET:
            tailStr = "/properties/set"; break;
        case PROP_SET_RES:
            tailStr = "/properties/set/response"; break;
        case PROP_GET:
            tailStr = "/properties/get"; break;
        case PROP_GET_RES:
            tailStr = "/properties/get/response"; break;
        case PROP_POST:
            tailStr = "/properties/post"; break;
        case FUNC_CALL:
            tailStr = "/functions/call"; break;
        case FUNC_CALL_RES:
            tailStr = "/functions/call/response"; break;
        case REG:
            tailStr = "/register"; break;
        case REG_RES:
            tailStr = "/register/response"; break;
        case THINGMODEL_POST:
            tailStr = "/thing-model/post"; break;
        case THINGMODEL_POST_RES:
            tailStr = "/thing-model/post/response"; break;
        default:
            tailStr = "";
    }
    if (servicKey && servicKey.length > 0) {
        retTopic = prefix + objName + "/" + deviceId + "/" + servicKey + tailStr
    } else {
        retTopic = prefix + objName + "/" + deviceId + tailStr
    }
    return retTopic
}

// 消息主题解析
function Parse(inTopic: string): Topic {
    let isService: boolean = true
    let ii: number
    let retTopic: Topic = {
        retain: "",
        version: "",
        objName: "",
        deviceId: "",
        serviceKey: "",
        typeName: "",
        command: "",
        response: "",
    };
    let strArr: string[] = (inTopic || "").split("/");
    let lenArr: number = strArr.length;
    if (lenArr >= minParts && lenArr <= maxParts) {
        // retain
        retTopic.retain = strArr[0]

        // version
        retTopic.version = strArr[1]

        // object namespace
        retTopic.objName = strArr[2]

        // deviceId
        retTopic.deviceId = strArr[3]

        // 判断是否含有service key
        if (strArr[4] == TYPE_NAME_PROP) {
            isService = false
        } else if (strArr[4] == TYPE_NAME_EVT) {
            isService = false
        } else if (strArr[4] == TYPE_NAME_FUNC) {
            isService = false
        } else if (strArr[4] == TYPE_NAME_REG) {
            isService = false
        } else if (strArr[4] == TYPE_NAME_AUTH) {
            isService = false
        } else if (strArr[4] == TYPE_NAME_HB) {
            isService = false
        } else if (strArr[4] == TYPE_NAME_THMD) {
            isService = false
        } else {
        }
        ii = 4
        if (isService) {
            retTopic.serviceKey = strArr[ii]
            ii++
        }

        // type name
        retTopic.typeName = strArr[ii]
        ii++

        // command
        if (lenArr > ii) {
            retTopic.command = strArr[ii]
            ii++

            // response
            if (lenArr > ii) {
                retTopic.response = strArr[ii]
            }
        }
    }
    return retTopic
}

export default {
    Get,
    Parse,

    TYPE_NAME_PROP,
    TYPE_NAME_EVT,
    TYPE_NAME_FUNC,
    TYPE_NAME_REG,
    TYPE_NAME_AUTH,
    TYPE_NAME_HB,
    TYPE_NAME_THMD,

    AUTH_POST,
    AUTH_POST_RES,
    THINGMODEL_POST,
    THINGMODEL_POST_RES,
    EVT_ONLINE,
    EVT_OFFLINE,
    HB_POST,
    HB_POST_RES,
    HB_SET,
    EVT_POST,
    PROP_SET,
    PROP_SET_RES,
    PROP_GET,
    PROP_GET_RES,
    PROP_POST,
    FUNC_CALL,
    FUNC_CALL_RES,
    REG,
    REG_RES,
}