import log4js from 'log4js'

// 文件名称
let logFileName = "joylinkSDK";
// log4js配置
log4js.configure({
    appenders: {
        console: {//记录器1:输出到控制台
            type: 'console',
        },
        log_file: {//记录器2：输出到文件
            type: 'file',
            filename: __dirname + `/../../logs/${logFileName}.log`, // 文件目录，当目录文件或文件夹不存在时，会自动创建
            maxLogSize: 20971520, // 20M. 文件最大存储空间（byte），当文件内容超过文件存储空间会自动生成一个文件test.log.1的序列自增长的文件
            backups: 3, // default value = 5.当文件内容超过文件存储空间时，备份文件的数量
            //compress : true, // default false.是否以压缩的形式保存新文件,默认false。如果true，则新增的日志文件会保存在gz的压缩文件内，并且生成后将不被替换，false会被替换掉
            encoding: 'utf-8', // default "utf-8"，文件的编码
        },
        date_file: {//：记录器3：输出到日期文件
            type: "dateFile",
            filename: __dirname + `/../../logs/${logFileName}`, // 您要写入日志文件的路径
            alwaysIncludePattern: true, // （默认为false） - 将模式包含在当前日志文件的名称以及备份中
            daysToKeep: 10, // 时间文件 保存多少天，距离当前天daysToKeep以前的log将被删除
            //compress : true, //（默认为false） - 在滚动期间压缩备份文件（备份文件将具有.gz扩展名）
            pattern: "yyyyMMdd.log",//（可选，默认为.yyyy-MM-dd） - 用于确定何时滚动日志的模式。格式:.yyyy-MM-dd-hh:mm:ss.log
            encoding: 'utf-8', // default "utf-8"，文件的编码
        },
        error_file: {//：记录器4：输出到error log
            type: "dateFile",
            filename: __dirname + `/../../logs/${logFileName}_error`, // 您要写入日志文件的路径
            alwaysIncludePattern: true, // （默认为false） - 将模式包含在当前日志文件的名称以及备份中
            daysToKeep: 10, // 时间文件 保存多少天，距离当前天daysToKeep以前的log将被删除
            //compress : true, // （默认为false） - 在滚动期间压缩备份文件（备份文件将具有.gz扩展名）
            pattern: "yyyyMMdd.log", //（可选，默认为.yyyy-MM-dd） - 用于确定何时滚动日志的模式。格式:.yyyy-MM-dd-hh:mm:ss.log
            encoding: 'utf-8', // default "utf-8"，文件的编码
            // compress: true, // 是否压缩
        }
    },
    categories: {
        develop: { appenders: ['console'], level: 'debug' }, // 开发环境  输出到控制台
        debug: { appenders: ['console', 'log_file'], level: 'debug' }, // 调试环境 输出到log文件和控制台    
        default: { appenders: ['console', 'date_file', 'log_file'], level: 'info' }, // 默认log类型，输出到控制台 log文件 log日期文件 且登记大于info即可
        production: { appenders: ['date_file'], level: 'warn' },  // 生产环境 log类型 只输出到按日期命名的文件，且只输出警告以上的log
        error_log: { appenders: ['console', 'error_file'], level: 'error' } // error 等级log 单独输出到error文件中 任何环境的errorlog 将都以日期文件单独记录
    },
});

/** logger枚举值 */
export enum Category {
    /** 开发环境: develop */
    DEVELOP = "develop",
    /** 调试环境: debug */
    DEBUG = "debug",
    /** 默认: default */
    DEFAULT = "default",
    /** 生产环境: production */
    PRODUCTION = "production"
}

export interface Log4jInte {
    logger?: log4js.Logger;

    trace(message: any, ...args: any[]): void;

    debug(message: any, ...args: any[]): void;

    info(message: any, ...args: any[]): void;

    warn(message: any, ...args: any[]): void;

    error(message: any, ...args: any[]): void;

    fatal(message: any, ...args: any[]): void;

    mark(message: any, ...args: any[]): void;
}

class Log4j implements Log4jInte {
    logger!: log4js.Logger;

    constructor(category?: Category) {
        this.logger = category ?
            log4js.getLogger(category)
            :
            log4js.getLogger(Category.DEBUG);
    }

    trace(message: any, ...args: any[]): void {
        this.logger.trace(message, ...args);
    }

    debug(message: any, ...args: any[]): void {
        this.logger.debug(message, ...args);
    }

    info(message: any, ...args: any[]): void {
        this.logger.info(message, ...args);
    }

    warn(message: any, ...args: any[]): void {
        this.logger.warn(message, ...args);
    }

    error(message: any, ...args: any[]): void {
        errLogger.error(message, ...args);
    }

    fatal(message: any, ...args: any[]): void {
        errLogger.fatal(message, ...args);
    }

    mark(message: any, ...args: any[]): void {
        errLogger.mark(message, ...args);
    }

}

// 错误日志
const errLogger = log4js.getLogger("error_log");

const logMap: { [key: string]: Log4jInte } = {}
logMap[Category.DEBUG] = new Log4j(Category.DEBUG)


/** 根据环境获取不同日志输出，默认 DEBUG */
export function getLogger(category?: Category): Log4jInte {
    if (!category) {
        return logMap[Category.DEBUG]
    }
    if (!logMap[category]) {
        logMap[category] = new Log4j(category)
    }
    return logMap[category]
}

// 默认日志
export const logger = getLogger()








